@@ -1,8 +1,8 @@
-use core::ops::{Add, Sub, Mul, Div, Neg};
-use num_traits::{FromPrimitive, ToPrimitive, RefNum, NumRef, Signed, Zero, One, CheckedAdd, CheckedMul};
+use core::ops::{Add, Sub, Mul, Div, Neg, AddAssign};
+use num_traits::{Num, FromPrimitive, ToPrimitive, RefNum, NumRef, Signed, Zero, One, CheckedAdd, CheckedMul};
 use num_integer::{Integer, Roots, sqrt};
 use std::fmt;
-use crate::traits::{Irrational, FromSqrt, Approximation, RationalApproximation, WithSigned, WithUnsigned};
+use crate::traits::{FromSqrt, Approximation, Computable, WithSigned, WithUnsigned};
 use crate::cont_frac::ContinuedFraction;
 
 use num_rational::Ratio;
@@ -362,7 +362,7 @@ where for<'r> &'r T: RefNum<T>, for<'r> &'r U: RefNum<U> {
 // Reference: http://www.numbertheory.org/courses/MP313/lectures/lecture17/page5.html
 //            http://www.numbertheory.org/gnubc/surd
 // assumes positive surd, parameter `neg` determines the sign of the fraction
-fn quadsurd_to_contfrac<T: QuadraticSurdBase + WithUnsigned<Unsigned = U>, U: Integer>
+fn quadsurd_to_contfrac<T: QuadraticSurdBase + WithUnsigned<Unsigned = U> + AddAssign, U>
 (a: T, b: T, c: T, r: T, neg: bool) -> ContinuedFraction<U>
 where for<'r> &'r T: RefNum<T> {
     debug_assert!(!c.is_zero() && r >= T::zero());
@@ -376,23 +376,24 @@ where for<'r> &'r T: RefNum<T> {
     }
 
     // find the reduced form and aperiodic coefficients
-    let mut a_coeffs: Vec<U> = Vec::new();
+    let mut a_coeffs: Vec<T> = Vec::new();
     let rd = d.sqrt();
-    while !(q > T::zero() && p > T::zero() && p <= rd && rd < (&p+&q) && (&q-&p) <= rd) {
+    while a_coeffs.len() == 0 || // ensure that we have a first coefficient
+          !(p <= rd && rd < (&p+&q) && (&q-&p) <= rd) {
         let a = (&rd + &p).div_floor(&q);
         p = &a*&q - p;
         q = (&d - &p*&p) / q;
-        a_coeffs.push(a.to_unsigned());
+        a_coeffs.push(a);
     }
 
     // find the periodic coefficients
-    let mut p_coeffs: Vec<U> = Vec::new();
+    let mut p_coeffs: Vec<T> = Vec::new();
     let (init_p, init_q) = (p.clone(), q.clone());
     loop {
         let a = (&rd + &p).div_floor(&q);
         p = &a*&q - p;
         q = (&d - &p*&p) / q;
-        p_coeffs.push(a.to_unsigned());
+        p_coeffs.push(a);
         if p == init_p && q == init_q { break }
     }
 
@@ -400,7 +401,7 @@ where for<'r> &'r T: RefNum<T> {
 }
 
 #[cfg(not(feature="complex"))]
-impl<T: QuadraticSurdBase + WithUnsigned<Unsigned = U>, U: Integer>
+impl<T: QuadraticSurdBase + WithUnsigned<Unsigned = U> + AddAssign, U>
 From<QuadraticSurd<T>> for ContinuedFraction<U>
 where for<'r> &'r T: RefNum<T> {
     fn from(s: QuadraticSurd<T>) -> Self {
@@ -419,13 +420,13 @@ impl<T: Integer> TryFrom<QuadraticSurd<T>> for ContinuedFraction<T> {
     }
 }
 
-impl<T: QuadraticSurdBase + CheckedAdd + WithUnsigned<Unsigned = U>,
+impl<T: QuadraticSurdBase + CheckedAdd + AddAssign + WithUnsigned<Unsigned = U>,
      U: Integer + Clone + CheckedAdd + CheckedMul + WithSigned<Signed = T>>
-RationalApproximation<T> for QuadraticSurd<T>
+     Computable<T> for QuadraticSurd<T>
 where for<'r> &'r T: RefNum<T>{
     #[cfg(not(feature="complex"))]
-    fn approx_rational(&self, limit: &T) -> Approximation<Ratio<T>> {
-        ContinuedFraction::<U>::from(self.clone()).approx_rational(limit)
+    fn approximated(&self, limit: &T) -> Approximation<Ratio<T>> {
+        ContinuedFraction::<U>::from(self.clone()).approximated(limit)
     }
 }
 
@@ -633,9 +634,6 @@ where for<'r> &'r T: RefNum<T> {
     }
 }
 
-impl<T: QuadraticSurdBase + ToPrimitive> Irrational for QuadraticSurd<T>
-where for<'r> &'r T: RefNum<T> {}
-
 #[derive(Copy, Clone, Debug, PartialEq)]
 pub struct FromSqrtError {
     kind: SqrtErrorKind
@@ -684,7 +682,7 @@ where for<'r> &'r T: RefNum<T> {
     }
 }
 
-impl<T: QuadraticSurdBase + CheckedMul + std::fmt::Debug> FromSqrt<QuadraticSurd<T>> for QuadraticSurd<T>
+impl<T: QuadraticSurdBase + CheckedMul> FromSqrt<QuadraticSurd<T>> for QuadraticSurd<T>
 where for<'r> &'r T: RefNum<T> {
     type Error = FromSqrtError;
 
@@ -823,6 +821,10 @@ mod tests {
 
     #[test]
     fn conversion_between_contfrac() {
+        let cf_phi = ContinuedFraction::<u32>::new(vec![1], vec![1], false);
+        assert_eq!(QuadraticSurd::from(cf_phi.clone()), PHI);
+        assert_eq!(ContinuedFraction::from(PHI), cf_phi);
+
         let cf_sq2 = ContinuedFraction::<u32>::new(vec![1], vec![2], false);
         let surd_sq2 = QuadraticSurd::new(0, 1, 1, 2);
         assert_eq!(QuadraticSurd::from(cf_sq2.clone()), surd_sq2);
